#!perl

use strict;
use Getopt::Long;
use MaterialsScript qw(:all);

my $doc_xsd = $Documents{"structure.xsd"};
my $doc_txt = $Documents{"Charges.txt"};

my $atoms = $doc_xsd->AsymmetricUnit->Atoms;
my $atom_cnt = 1;
foreach my $atom (@$atoms) 
{
	$atom->Name = $atom->ElementSymbol.$atom_cnt;
	$atom_cnt++;
}

my $atoms = $doc_xsd->UnitCell->Atoms;
my @mult_charges;

for (my $i=0; $i<$doc_txt->Lines->Count; ++$i)
#for (my $i=0; $i<20; ++$i)
{
	#print $doc_txt->Lines($i);
	my @values = split(';', $doc_txt->Lines($i));
	foreach my $atom (@$atoms) 
	{
		if(($atom->ElementSymbol eq $values[0]))
		{
			my $charges_X = norm_coord($values[1]);
			my $charges_y = norm_coord($values[2]);
			my $charges_z = norm_coord($values[3]);
			
			my $xsd_X = norm_coord($atom->XYZ->X);
			my $xsd_y = norm_coord($atom->XYZ->Y);
			my $xsd_z = norm_coord($atom->XYZ->Z);
			
			my $dist = sqrt((($charges_X-$xsd_X)**2)+(($charges_y-$xsd_y)**2)+(($charges_z-$xsd_z)**2));
			if($dist<0.01)
			{
				my @in_array = ($atom->Name, ($values[4]+0.00));
				push(@mult_charges, [@in_array]);
			}
		}
	}
}

foreach my $atom (@$atoms) 
{
	my $charge = 0;
	my $charges_cnt = 0;
	
	for (my $i=0; $i<=$#mult_charges; ++$i)
	{
		if(($mult_charges[$i][0] eq $atom->Name))
		{
			$charge += $mult_charges[$i][1];
			$charges_cnt += 1;
		}	
	}
	print $atom->Name.": ".$charges_cnt."x\n";
	$atom->Charge = $charge/$charges_cnt;
}

sub is_similar
{
	if((($_[0]+0.01)>$_[1]+0) && (($_[0]-0.01)<$_[1]+0))
	{
		return 1;
	}
}

sub norm_coord
{
	if(($_[0]+0)<0)
	{
		return ($_[0]+0)+$doc_xsd->SymmetryDefinition->LengthA;
	}
	if(($_[0]+0)>$doc_xsd->SymmetryDefinition->LengthA)
	{
		return ($_[0]+0)-$doc_xsd->SymmetryDefinition->LengthA;
	}
	if((($_[0]+0)<=$doc_xsd->SymmetryDefinition->LengthA) and (($_[0]+0)>=0))
	{
		return ($_[0]+0);
	}
}