#!perl

use MaterialsScript qw(:all);

my $source_filename = "Zr-PPPP_Fd-3_ChargeEq";
my $doc = $Documents{($source_filename.".xsd")};
my $cluster_doc = Documents->New($source_filename."_cluster.xsd");
$doc->MakeP1;

my $atoms = $doc->UnitCell->Atoms;
my $start;
my $index = 0;
foreach my $atom (@$atoms) 
{
	if($atom->Name =~ m/_/) #in XSD zwei Cluster suchen, die komplett in PIZOF-EZ sind, ein C-Atom in mittlerem Ring w�hlen, an dem ein Seitenarm h�ngt, dem Atomnamen ein _ vorranstellen (z.B. C67 => _C67), speichern 
	{
			$start = $index;
	}
	$index++;
}

my @attached;
my @temp;
my @last;
push(@attached, $start);
push(@last, $start);
for (my $i = 0; $i <= 15; $i++) #Wichtiger Wert (15), damit an Clustern keine Substituenten erstellt werden, die kompletten Phenylring enstprechen (au�er Verkn�pfendem Linker)
{
	undef @temp;
	foreach my $atom (@last)
	{
		push(@temp, neighbours($atom));
	}
	undef @last;
	foreach my $atom_index (@temp)
	{
		if(grep { $_ eq $atom_index } @attached)
		{
			print "duplicate\n";
		}
		else
		{
			push(@attached, $atom_index);
			push(@last, $atom_index);
		}
	}	
	@attached = uniq(@attached);
}

foreach my $atom_index (@attached)
{
	print $atom_index." -> ".$atoms->[$atom_index]->ElementSymbol."\n";
	$cluster_doc->CreateAtom($atoms->[$atom_index]->ElementSymbol, Point(X => $atoms->[$atom_index]->XYZ->X, Y => $atoms->[$atom_index]->XYZ->Y, Z => $atoms->[$atom_index]->XYZ->Z), ["Name" => ($atoms->[$atom_index]->ElementSymbol.$atom_index)]);
}

foreach my $atom (@$atoms) 
{
	if($atom->Name =~ m/_/)
	{
			$atom->Name = substr($atom->Name, 1, (length($atom->Name)-1));
	}
}

Tools->BondCalculation->Bonds->Calculate($cluster_doc);

my $atom_index;
my $atoms = $cluster_doc->Atoms;
my $atoms_cryst = $doc->UnitCell->Atoms;
my @attached_benz;
my @attached;
my @temp;
my @last;
my $name = "Benz";
foreach my $atom (@$atoms) 
{
	if($atom->ElementSymbol eq "C")
	{
		undef @attached_benz;
		undef @last;
		undef @temp;
		my $check_o = 0;
		my $attached_atoms = $atom->AttachedAtoms;
		foreach my $atom_att (@$attached_atoms) 
		{
			if($atom_att->ElementSymbol eq "O")
			{
				$check_o++;
			}
		}
		if($check_o==2)
		{
			$atom_index = substr($atom->Name,1,(length($atom->Name)-1));
			my $carb_c = $doc->UnitCell->Atoms->[$atom_index];
			push(@carb_array, $carb_index);
			$attached_atoms = $carb_c->AttachedAtoms;
			my $first_c;
			foreach my $atom_att (@$attached_atoms) 
			{
				if($atom_att->ElementSymbol eq "C")
				{
					if(find_close($atom_att->X, $atom_att->Y, $atom_att->Z)=="ok")
					{
						$cluster_doc->CreateAtom($atom_att->ElementSymbol, Point(X => $atom_att->XYZ->X, Y => $atom_att->XYZ->Y, Z => $atom_att->XYZ->Z), ["Name" => $name]);
					}
					$first_c = $atom_att;
				}
			}
			$attached_atoms = $first_c->AttachedAtoms;
			my $final_check = 0;
			foreach my $atom_att (@$attached_atoms) 
			{
				if(($carb_c->XYZ->X!=$atom_att->XYZ->X) and ($carb_c->XYZ->Y!=$atom_att->XYZ->Y) and ($carb_c->XYZ->Z!=$atom_att->XYZ->Z))
				{	
					if(find_close($atom_att->X, $atom_att->Y, $atom_att->Z)=="ok")
					{
						$cluster_doc->CreateAtom($atom_att->ElementSymbol, Point(X => $atom_att->XYZ->X, Y => $atom_att->XYZ->Y, Z => $atom_att->XYZ->Z), ["Name" => $name]);
					}
					my $attached_atoms2 = $atom_att->AttachedAtoms;
					foreach my $atom_att2 (@$attached_atoms2) 
					{
						if(($first_c->XYZ->X!=$atom_att2->XYZ->X) and ($first_c->XYZ->Y!=$atom_att2->XYZ->Y) and ($first_c->XYZ->Z!=$atom_att2->XYZ->Z))
						{
							if(find_close($atom_att2->X, $atom_att2->Y, $atom_att2->Z)=="ok")
							{
								$cluster_doc->CreateAtom($atom_att2->ElementSymbol, Point(X => $atom_att2->XYZ->X, Y => $atom_att2->XYZ->Y, Z => $atom_att2->XYZ->Z), ["Name" => $name]);
							}
							my $attached_atoms3 = $atom_att2->AttachedAtoms;
							foreach my $atom_att3 (@$attached_atoms3) 
							{
								if(($atom_att2->XYZ->X!=$atom_att3->XYZ->X) and ($atom_att2->XYZ->Y!=$atom_att3->XYZ->Y) and ($atom_att2->XYZ->Z!=$atom_att3->XYZ->Z))
								{
									if(find_close($atom_att3->X, $atom_att3->Y, $atom_att3->Z)=="ok")
									{
										$cluster_doc->CreateAtom($atom_att3->ElementSymbol, Point(X => $atom_att3->XYZ->X, Y => $atom_att3->XYZ->Y, Z => $atom_att3->XYZ->Z), ["Name" => $name]);
									}
								}
							}
						}
					}
				}
			}
		}
	}
}

Tools->BondCalculation->Bonds->Calculate($cluster_doc);
my $atoms = $cluster_doc->Atoms;
foreach my $atom (@$atoms) 
{
	if(($atom->ElementSymbol eq "C") and ($atom->AttachedAtoms->Count == 2))
	{
		#print "numbonds = 2 \n";
		my $check_ring = 0;
		my @c_vec = ();
		my $attached_atoms = $atom->AttachedAtoms;
		foreach my $atom_att (@$attached_atoms) 
		{
			#print $atom_att->NumBonds."\n";
			if($atom_att->ElementSymbol eq "C")
			{
				my $h_check = 0;
				my $attached_atoms2 = $atom_att->AttachedAtoms;
				foreach my $atom_att2 (@$attached_atoms2) 
				{
					if($atom_att2->ElementSymbol eq "H")
					{
						$h_check++;
					}
				}
				if($h_check==1)
				{
					#print "check_ring plus \n";
					my @atom_vec = (($atom_att->XYZ->X-$atom->XYZ->X),($atom_att->XYZ->Y-$atom->XYZ->Y),($atom_att->XYZ->Z-$atom->XYZ->Z));
					push(@c_vec, [@atom_vec]);
					$check_ring++;
				}
			}
		}
		if($check_ring == 2)
		{
			my @h_vec = (($c_vec[0][0]+$c_vec[1][0]),($c_vec[0][1]+$c_vec[1][1]),($c_vec[0][2]+$c_vec[1][2])); 
			my $hev_vec_len = sqrt(($h_vec[0]**2)+($h_vec[1]**2)+($h_vec[2]**2));
			my @h_vec_norm = ((-1*($h_vec[0]/$hev_vec_len)), (-1*($h_vec[1]/$hev_vec_len)), (-1*($h_vec[2]/$hev_vec_len)));
			my @h_vec_create = ((1.08087*$h_vec_norm[0]),(1.08087*$h_vec_norm[1]),(1.08087*$h_vec_norm[2]));
			$cluster_doc->CreateAtom("H", Point(X => ($atom->XYZ->X+$h_vec_create[0]), Y => ($atom->XYZ->Y+$h_vec_create[1]), Z => ($atom->XYZ->Z+$h_vec_create[2])), ["Name" => "H_Benz"]);
			#$cluster_doc->CreateAtom("Si", Point(X => ($atom->XYZ->X+$h_vec_norm[0]), Y => ($atom->XYZ->Y+$h_vec_norm[1]), Z => ($atom->XYZ->Z+$h_vec_norm[2])), ["Name" => "H_Benz"]);
			#$cluster_doc->CreateAtom("Si", Point(X => ($atom->XYZ->X+0.1), Y => ($atom->XYZ->Y+0.1), Z => ($atom->XYZ->Z+0.1)), ["Name" => "H_Benz"]);
			print "Hydrogen atom added.\n";
		}
		print "---\n";
	}
}

Tools->BondCalculation->Bonds->Calculate($cluster_doc);

sub neighbours
{
	my $atoms = $doc->UnitCell->Atoms;
	my @attached_sub = ();
	my $attached_atoms = $atoms->[$_[0]]->AttachedAtoms;
	foreach my $atom_att (@$attached_atoms) 
	{
		my $index = 0;
		foreach my $atom (@$atoms) 
		{
			if(($atom->XYZ->X==$atom_att->XYZ->X) and ($atom->XYZ->Y==$atom_att->XYZ->Y) and ($atom->XYZ->Z==$atom_att->XYZ->Z))
			{
				push(@attached_sub, $index);
			}		
			$index++;
		}	
	}
	return @attached_sub;
}

sub find_close()
{
	my $out = "ok";
	for (my $i=0; $i<$cluster_doc->Atoms->Count; ++$i) 
	{
		my @dist_vec = (($cluster_doc->Atoms($i)->X-$_[0]),($cluster_doc->Atoms($i)->Y-$_[1]),($cluster_doc->Atoms($i)->Z-$_[2]));
		my $vec_length = sqrt(($dist_vec[0]**2)+($dist_vec[1]**2)+($dist_vec[2]**2));
		if ($vec_length<0.1)
		{
			$out = $i;
			break;
		}
	}
	return $out;
}

sub uniq { my %seen; grep !$seen{$_}++, @_ }