import sys
import os
import pylab
import numpy
import scipy
import scipy.optimize
import scipy.linalg
import scipy.integrate
import scipy.signal
import copy
import matplotlib.pyplot as plt
import matplotlib.mlab as ml

vdwRadius = {"H": 1.20, "D": 1.20, "He": 1.40, "Li": 1.82, "Be": 0.63, "B": 1.75, "C": 1.775, "N": 1.50, "O": 1.45, "F": 1.47, "Ne": 1.54, "Na": 2.27, "Mg": 1.73, "Al": 1.50, "Si": 2.10, "P": 1.90, "S": 1.80, "Cl": 1.75, "Ar": 1.88, "K": 2.75, "Ca": 1.95, "Sc": 1.32, "Ti": 1.95, "V": 1.06, "Cr": 1.13, "Mn": 1.19, "Fe": 1.26, "Co": 1.13, "Ni": 1.63, "Cu": 1.40, "Zn": 1.39, "Ga": 1.87, "Ge": 1.48, "As": 0.83, "Se": 1.90, "Br": 1.85, "Kr": 2.02, "Rb": 2.65, "Sr": 2.02, "Y": 1.61, "Zr": 1.42, "Nb": 1.33, "Mo": 1.75, "Tc": 2.00, "Ru": 1.20, "Rh": 1.22, "Pd": 1.63, "Ag": 1.72, "Cd": 1.58, "In": 1.93, "Sn": 2.17, "Sb": 1.12, "Te": 1.26, "I": 1.98, "Xe": 2.16, "Cs": 3.01, "Ba": 2.41, "La": 1.83, "Ce": 1.86, "Pr": 1.62, "Nd": 1.79, "Pm": 1.76, "Sm": 1.74, "Eu": 1.96, "Gd": 1.69, "Tb": 1.66, "Dy": 1.63, "Ho": 1.61, "Er": 1.59, "Tm": 1.57, "Yb": 1.54, "Lu": 1.53, "Hf": 1.40, "Ta": 1.22, "W": 1.26, "Re": 1.30, "Os": 1.58, "Ir": 1.22, "Pt": 1.72, "Au": 1.66, "Hg": 1.55, "Tl": 1.96, "Pb": 2.02, "Bi": 1.73, "Po": 1.21, "At": 1.12, "Rn": 2.30, "Fr": 3.24, "Ra": 2.57, "Ac": 2.12, "Th": 1.84, "Pa": 1.60, "U": 1.75, "Np": 1.71, "Pu": 1.67, "Am": 1.66, "Cm": 1.65, "Bk": 1.64, "Cf": 1.63, "Es": 1.62, "Fm": 1.61, "Md": 1.60, "No": 1.59, "Lr": 1.58}
molarAtomicMass = {"Ru": 102.91, "Pd": 106.4, "Pt": 195.09, "Ni": 58.71, "Mg": 24.305, "Na": 22.99, "Nb": 92.906, "Db": 268.0, "Ne": 20.179, "Li": 6.941, "Pb": 207.2, "Re": 128.207, "Tl": 204.37, "B": 10.81, "Ra": 226.03, "Rb": 85.468, "Ti": 47.9, "Rn": 222.0, "Cd": 112.41, "Po": 209.0, "Ta": 180.95, "Be": 9.0122, "Fr": 223.0, "Te": 127.6, "Ba": 137.33, "Os": 190.2, "La": 138.9, "Bh": 272.0, "Ge": 72.59, "Zr": 91.224, "Tc": 97.0, "Fe": 55.847, "Br": 79.904, "Sr": 87.62, "Hf": 178.49, "Hg": 200.59, "He": 4.0026, "C": 12.011, "Cl": 35.453, "Rf": 265.0, "P": 30.974, "F": 18.998, "I": 126.9, "H": 1.0079, "Mo": 95.94, "v": 50.941, "Ac": 227.0, "O": 15.999, "N": 14.007, "Kr": 83.8, "Si": 28.086, "Sn": 118.69, "W": 183.84, "Y": 88.906, "Sb": 121.75, "Bi": 206.98, "Al": 26.982, "Sg": 271.0, "Se": 78.96, "Sc": 44.955912, "Zn": 65.38, "Co": 58.933, "Ag": 107.87, "Mt": 276.0, "k": 39.096, "Ir": 192.22, "S": 32.06, "Xe": 131.3, "Mn": 54.938, "As": 74.922, "Ar": 39.948, "Au": 196.97, "At": 210.0, "Ga": 69.72, "Hs": 227.0, "Cs": 132.91, "Cr": 51.996, "Ca": 40.08, "Cu": 63.546, "In": 114.82}
avogadroConstant = 6.02214129*10**23 #Avodadro constant (one mol)
boltzmannConstant = 1.3806488*10**-23 # Boltzmann constant in J/K
idealGasConstant = avogadroConstant*boltzmannConstant # Ideal gas constant in J/mol*K
hplanck = 6.62606957*10**-34 # Planck's constant in J/s or m^2*kg/s

binXMax = 0.0
binYMax = 0.0
binZMax = 0.0
binXMin = 1000.0
binYMin = 1000.0
binZMin = 1000.0
binNumberPerDimensionX = 100 #60
binNumberPerDimensionY = 100 #60
binNumberPerDimensionZ = 100 #40
	
def readMDfile(filename):
	mdFileData = []
	ins = open(filename, "r")
	isNewFrame = True
	mdFileDataLine = []
	for line in ins:
		if isNewFrame and ("------" not in line):
			splitData = line.split(" ")
			lineArray = [splitData[0], float(splitData[1]), float(splitData[2]), float(splitData[3]), float(splitData[4])]
			mdFileDataLine.append(lineArray)
		if ("------" in line) and isNewFrame:
			isNewFrame = True
			mdFileData.append(mdFileDataLine)
			mdFileDataLine = []
	ins.close()
	return mdFileData	
	
def plotDensityData(analyzedMdData, zStart, png_name=None):
	global binXMax, binYMax, binZMax, binXMin, binYMin, binZMin, binNumberPerDimensionX, binNumberPerDimensionY, binNumberPerDimensionZ, vdwRadius
	x = []
	y = []
	z = []
	
	binSizeX = (binXMax-binXMin)/binNumberPerDimensionX
	binSizeY = (binYMax-binYMin)/binNumberPerDimensionY
	binSizeZ = (binZMax-binZMin)/binNumberPerDimensionZ
	
	for i in range(binNumberPerDimensionX):
		for j in range(binNumberPerDimensionY):
			x.append(binSizeX*i)
			y.append(binSizeY*j)
			z.append(analyzedMdData[i][j][zStart])
			
	maxDensity = 0.0
	for i in range(binNumberPerDimensionX):
		for j in range(binNumberPerDimensionY):
			for k in range(len(analyzedMdData[i][j])):
				if maxDensity<analyzedMdData[i][j][k]:
					maxDensity = analyzedMdData[i][j][k]
					
	for i in range(len(z)):
		z[i] = z[i]/maxDensity
					
	xmin,xmax,ymin,ymax = [min(x),max(x),min(y),max(y)]
	
	#size of 1 m grid
	nx = 500#CHANGE HERE
	ny = 500#CHANGE HERE
	
	# Generate a regular grid to interpolate the data.
	xi = np.linspace(xmin, xmax, nx)
	yi = np.linspace(ymin, ymax, ny)
	xi, yi = np.meshgrid(xi, yi) 
	
	 
	
	# Interpolate the values of z for all points in the rectangular grid
	# Method 1 - Interpolate by matplotlib delaunay triangularizatio and nearest neigh. PLEASE NOTE! THIS FAILS QUITE OFTEN (http://matplotlib.org/api/mlab_api.html#matplotlib.mlab.griddata) But there might be a solution - install mpl_toolkits.natgrid (http://matplotlib.org/mpl_toolkits/)
	zi = ml.griddata(x,y,z,xi,yi,interp='nn') 
	
	plt.pcolormesh(xi, yi, zi, cmap = plt.get_cmap('jet'))
	
	cb = plt.colorbar() 
	cb.set_clim(0, 1.0)
	#plt.scatter(x_list, y_list, marker = 'o', c = 'b', s = 5, zorder = 10)
	plt.xlim(xmin, xmax)
	plt.ylim(ymin, ymax)
	if png_name!=None:
		plt.savefig(png_name, format='png', dpi=400)
		plt.close()
	else:
		plt.show()

def analyzeBinsVdw(mdFileData):
	global binXMax, binYMax, binZMax, binXMin, binYMin, binZMin, binNumberPerDimensionX, binNumberPerDimensionY, binNumberPerDimensionZ, vdwRadius
	for i in range(len(mdFileData)):
		for j in range(len(mdFileData[i])):
			if mdFileData[i][j][1]>binXMax:
				binXMax = mdFileData[i][j][1]
			if mdFileData[i][j][2]>binYMax:
				binYMax = mdFileData[i][j][2]
			if mdFileData[i][j][3]>binZMax:
				binZMax = mdFileData[i][j][3]
			if mdFileData[i][j][1]<binXMin:
				binXMin = mdFileData[i][j][1]
			if mdFileData[i][j][2]<binYMin:
				binYMin = mdFileData[i][j][2]
			if mdFileData[i][j][3]<binZMin:
				binZMin = mdFileData[i][j][3]
	binSizeX = (binXMax-binXMin)/binNumberPerDimensionX
	binSizeY = (binYMax-binYMin)/binNumberPerDimensionY
	binSizeZ = (binZMax-binZMin)/binNumberPerDimensionZ
	binArray = numpy.zeros((binNumberPerDimensionX, binNumberPerDimensionY, binNumberPerDimensionZ))
	for i in range(len(mdFileData)):
		print "Frame: "+str(i+1)
		for j in range(len(mdFileData[i])):
			#print "Atom: "+str(j+1)
			if (mdFileData[i][j][1]<=binXMax) and (mdFileData[i][j][2]<=binYMax) and (mdFileData[i][j][3]<=binZMax):
				if (mdFileData[i][j][1]>=binXMin) and (mdFileData[i][j][2]>=binYMin) and (mdFileData[i][j][3]>=binZMin):
					binCenter = numpy.empty((3,1))
					binEdge = numpy.empty((3,1))
					for k in range(0, binNumberPerDimensionX):
						if mdFileData[i][j][1]<binXMin+((k+1)*binSizeX):
							binCenter[0] = binXMin+((k+1)*binSizeX)-0.5*binSizeX
							binEdge[0] = binXMin+((k+1)*binSizeX)-binSizeX
							break
					for l in range(0, binNumberPerDimensionY):
						if mdFileData[i][j][2]<binYMin+((l+1)*binSizeY):
							binCenter[1] = binYMin+((l+1)*binSizeY)-0.5*binSizeY
							binEdge[1] = binYMin+((l+1)*binSizeY)-binSizeY
							break
					for m in range(0, binNumberPerDimensionZ):
						if mdFileData[i][j][3]<binZMin+((m+1)*binSizeZ):
							binCenter[2] = binZMin+((m+1)*binSizeZ)-0.5*binSizeZ
							binEdge[2] = binZMin+((m+1)*binSizeZ)-binSizeZ
							break
					if numpy.sum(binCenter)>0:
						currAtomVdwRadius = vdwRadius[mdFileData[i][j][0]]
						currAtomMass = molarAtomicMass[mdFileData[i][j][0]]
						currAtomCenter = numpy.array([mdFileData[i][j][1], mdFileData[i][j][2], mdFileData[i][j][3]])
						kPervDw = int(currAtomVdwRadius/binSizeX)
						lPervDw = int(currAtomVdwRadius/binSizeY)
						mPervDw = int(currAtomVdwRadius/binSizeZ)
						kmin = k-kPervDw
						if kmin<=0: kmin=0
						kmax = k+kPervDw
						if kmax>=binNumberPerDimensionX: kmax=binNumberPerDimensionX
						#print [kmin, k, kmax]
						lmin = l-lPervDw
						if lmin<=0: lmin=0
						lmax = l+lPervDw
						if lmax>=binNumberPerDimensionY: lmax=binNumberPerDimensionY
						#print [lmin, l, lmax]
						mmin = m-mPervDw
						if mmin<=0: mmin=0
						mmax = m+mPervDw
						if mmax>=binNumberPerDimensionZ: mmax=binNumberPerDimensionZ
						#print [mmin, m, mmax]
						binCount = 0
						for n in range(kmin, kmax):
							for o in range(lmin, lmax):
								for p in range(mmin, mmax):
									if numpy.linalg.norm(currAtomCenter-numpy.array([binXMin+binSizeX*n, binYMin+binSizeY*o, binZMin+binSizeZ*p]))<=currAtomVdwRadius:
										binCount += 1
						partialMassCount = currAtomMass/binCount
						#binArray[k][l][m] += partialVdwCount
						for n in range(kmin, kmax):
							for o in range(lmin, lmax):
								for p in range(mmin, mmax):
									if numpy.linalg.norm(currAtomCenter-numpy.array([binXMin+binSizeX*n, binYMin+binSizeY*o, binZMin+binSizeZ*p]))<=currAtomVdwRadius:
										binArray[n][o][p] += partialMassCount
	return binArray

def buildPovFile(binarray, povTmpFilename, targetPovFilename):
	global binXMax, binYMax, binZMax, binXMin, binYMin, binZMin, binNumberPerDimensionX, binNumberPerDimensionY, binNumberPerDimensionZ, vdwRadius
	
	binSizeX = (binXMax-binXMin)/binNumberPerDimensionX
	binSizeY = (binYMax-binYMin)/binNumberPerDimensionY
	binSizeZ = (binZMax-binZMin)/binNumberPerDimensionZ	
	
	binXYZbottomList = []
	binValueList = []
	for i in range(binarray.shape[0]):
		for j in range(binarray.shape[1]):
			for k in range(binarray.shape[2]):
				binXCenter = None
				binYCenter = None
				binZCenter = None
				if i<1:
					binXbottom = binXMin
				else:
					binXbottom = binXMin+(i)*binSizeX
				if j<1:
					binYbottom = binYMin
				else:
					binYbottom = binYMin+(j)*binSizeY
				if k<1:
					binZbottom = binZMin
				else:
					binZbottom = binZMin+(k)*binSizeZ
					
				#print str(binXCenter)+" "+str(binYCenter)+" "+str(binZCenter)+" "+str(binarray[i][j][k])
				binXYZbottomList.append([binXbottom, binYbottom, binZbottom])
				binValueList.append(binarray[i][j][k])
				
	binXYZbottomArray = numpy.array(binXYZbottomList)
	binValueArray = numpy.array(binValueList)
	
	binXYZCenter = numpy.array([0.,0.,0.])
	for binXYZbottom in binXYZbottomArray:
		binXYZCenter += binXYZbottom
		
	binXYZCenter /= len(binXYZbottomArray)
	
	#print binXYZCenter	
	
	maxBinValue = max(binValueArray)
	binValueArray /= maxBinValue
	
	boxList = []
	for i in range(len(binValueArray)):
		newXYZarray = binXYZbottomList[i]-binXYZCenter
		if binValueArray[i]>0:
			boxList.append("  box { <0,0,0>, <"+str(binSizeX)+", "+str(binSizeY)+", "+str(binSizeZ)+"> pigment {color rgbt < 1, .0, .0, "+str(1-binValueArray[i])+">} translate <"+str(newXYZarray[0])+","+str(newXYZarray[1])+","+str(newXYZarray[2])+"> }")
	
	lineTopList = []
	lineDownList = []
	placeholderFound = False
	ins = open(povTmpFilename, "r")
	for line in ins:
		if "$placeholder$" in line:
			placeholderFound = True
		if "$placeholder$" not in line:
			if placeholderFound:
				lineDownList.append(line)
			else:
				lineTopList.append(line)
	ins.close()
	
	fo = open(targetPovFilename, "w")
	for lineTop in lineTopList:
		fo.write(lineTop)
	for box in boxList:
		fo.write(box+"\n")
	for lineDown in lineDownList:
		fo.write(lineDown)
	fo.close()
	
mdFileData =  readMDfile("filename.txt")
analyzedMdData = analyzeBinsVdw(mdFileData)
binCoordArray = numpy.array([binXMax, binYMax, binZMax, binXMin, binYMin, binZMin, binNumberPerDimensionX, binNumberPerDimensionY, binNumberPerDimensionZ])
#buildPovFile(analyzedMdData, "P5_test.tmp", "P5_test.pov")
numpy.save("filename.npy", analyzedMdData)
numpy.save("filename_bin_data.npy", binCoordArray)

for zStart in range(binNumberPerDimensionZ):
	plotDensityData(analyzedMdData, zStart, png_name=(str(zStart)+"_filename.png"))