import numpy as np
import matplotlib.pyplot as plt
import os
import math
import copy

vdwRadius = {"H": 1.20, "D": 1.20, "He": 1.40, "Li": 1.82, "Be": 0.63, "B": 1.75, "C": 1.775, "N": 1.50, "O": 1.45, "F": 1.47, "Ne": 1.54, "Na": 2.27, "Mg": 1.73, "Al": 1.50, "Si": 2.10, "P": 1.90, "S": 1.80, "Cl": 1.75, "Ar": 1.88, "K": 2.75, "Ca": 1.95, "Sc": 1.32, "Ti": 1.95, "V": 1.06, "Cr": 1.13, "Mn": 1.19, "Fe": 1.26, "Co": 1.13, "Ni": 1.63, "Cu": 1.40, "Zn": 1.39, "Ga": 1.87, "Ge": 1.48, "As": 0.83, "Se": 1.90, "Br": 1.85, "Kr": 2.02, "Rb": 2.65, "Sr": 2.02, "Y": 1.61, "Zr": 1.42, "Nb": 1.33, "Mo": 1.75, "Tc": 2.00, "Ru": 1.20, "Rh": 1.22, "Pd": 1.63, "Ag": 1.72, "Cd": 1.58, "In": 1.93, "Sn": 2.17, "Sb": 1.12, "Te": 1.26, "I": 1.98, "Xe": 2.16, "Cs": 3.01, "Ba": 2.41, "La": 1.83, "Ce": 1.86, "Pr": 1.62, "Nd": 1.79, "Pm": 1.76, "Sm": 1.74, "Eu": 1.96, "Gd": 1.69, "Tb": 1.66, "Dy": 1.63, "Ho": 1.61, "Er": 1.59, "Tm": 1.57, "Yb": 1.54, "Lu": 1.53, "Hf": 1.40, "Ta": 1.22, "W": 1.26, "Re": 1.30, "Os": 1.58, "Ir": 1.22, "Pt": 1.72, "Au": 1.66, "Hg": 1.55, "Tl": 1.96, "Pb": 2.02, "Bi": 1.73, "Po": 1.21, "At": 1.12, "Rn": 2.30, "Fr": 3.24, "Ra": 2.57, "Ac": 2.12, "Th": 1.84, "Pa": 1.60, "U": 1.75, "Np": 1.71, "Pu": 1.67, "Am": 1.66, "Cm": 1.65, "Bk": 1.64, "Cf": 1.63, "Es": 1.62, "Fm": 1.61, "Md": 1.60, "No": 1.59, "Lr": 1.58}
covRadius = {"H": 0.32, "He": 0.93, "Li": 1.23, "Be": 0.9, "B": 0.82, "C": 0.77, "N": 0.75, "O": 0.73, "F": 0.72, "Ne": 0.71, "Na": 1.53, "Mg": 1.36, "Al": 1.18, "Si": 1.1, "P": 1.06, "S": 1.02, "Cl": 0.99, "Ar": 0.98, "K": 2.03, "Ca": 1.74, "Sc": 1.44, "Ti": 1.32, "V": 1.22, "Cr": 1.18, "Mn": 1.17, "Fe": 1.17, "Co": 1.16, "Ni": 1.15, "Cu": 1.17, "Zn": 1.25, "Ga": 1.26, "Ge": 1.22, "As": 1.2, "Se": 1.16, "Br": 1.14, "Kr": 1.89, "Rb": 2.16, "Sr": 1.91, "Y": 1.62, "Zr": 1.45, "Nb": 1.34, "Mo": 1.3, "Tc": 1.27, "Ru": 1.25, "Rh": 1.25, "Pd": 1.28, "Ag": 1.34, "Cd": 1.41, "In": 1.44, "Sn": 1.41, "Sb": 1.4, "Te": 1.6, "I": 1.33, "Xe": 1.31, "Cs": 2.35, "Ba": 1.98, "La": 1.25, "Hf": 1.44, "Ta": 1.34, "W": 1.3, "Re": 1.28, "Os": 1.26, "Ir": 1.27, "Pt": 1.3, "Au": 1.34, "Hg": 1.49, "Tl": 1.48, "Pb": 1.47, "Bi": 1.46, "Po": 1.53, "At": 1.47, "Ce": 1.65, "Pr": 1.65, "Nd": 1.64, "Pm": 1.63, "Sm": 1.62, "Eu": 1.85, "Gd": 1.61, "Tb": 1.59, "Dy": 1.59, "Ho": 1.58, "Er": 1.57, "Tm": 1.56, "Yb": 1.7, "Lu": 1.56, "Th": 1.65, "U": 1.42, "Gh": 1}
atomicMass = {"Ru": 102.91, "Pd": 106.4, "Pt": 195.09, "Ni": 58.71, "Mg": 24.305, "Na": 22.99, "Nb": 92.906, "Db": 268.0, "Ne": 20.179, "Li": 6.941, "Pb": 207.2, "Re": 128.207, "Tl": 204.37, "B": 10.81, "Ra": 226.03, "Rb": 85.468, "Ti": 47.9, "Rn": 222.0, "Cd": 112.41, "Po": 209.0, "Ta": 180.95, "Be": 9.0122, "Fr": 223.0, "Te": 127.6, "Ba": 137.33, "Os": 190.2, "La": 138.9, "Bh": 272.0, "Ge": 72.59, "Zr": 91.224, "Tc": 97.0, "Fe": 55.847, "Br": 79.904, "Sr": 87.62, "Hf": 178.49, "Hg": 200.59, "He": 4.0026, "C": 12.011, "Cl": 35.453, "Rf": 265.0, "P": 30.974, "F": 18.998, "I": 126.9, "H": 1.0079, "Mo": 95.94, "v": 50.941, "Ac": 227.0, "O": 15.999, "N": 14.007, "Kr": 83.8, "Si": 28.086, "Sn": 118.69, "W": 183.84, "Y": 88.906, "Sb": 121.75, "Bi": 206.98, "Al": 26.982, "Sg": 271.0, "Se": 78.96, "Sc": 44.955912, "Zn": 65.38, "Co": 58.933, "Ag": 107.87, "Mt": 276.0, "k": 39.096, "Ir": 192.22, "S": 32.06, "Xe": 131.3, "Mn": 54.938, "As": 74.922, "Ar": 39.948, "Au": 196.97, "At": 210.0, "Ga": 69.72, "Hs": 227.0, "Cs": 132.91, "Cr": 51.996, "Ca": 40.08, "Cu": 63.546, "In": 114.82}

class Atom:
	def __init__(self, elementSymbol="Dummy", x=0.0, y=0.0, z=0.0):
		self.x = x
		self.y = y
		self.z = z
		self.elementSymbol = elementSymbol
		
class OrthorhombicCrystal:
	def __init__(self, a=0.0, b=0.0, c=0.0, alpha=90.0, beta=90.0, gamma=90.0):
		self.a = a
		self.b = b
		self.c = c
		self.alpha = alpha
		self.beta = beta
		self.gamma = gamma
		self.atoms = []
		
	def addAtom(self, elementSymbol="Dummy", x=0.0, y=0.0, z=0.0):
		x, y, z = self.wrapIntoBox(x, y, z)
		self.atoms.append(Atom(elementSymbol, x, y, z))
		return self.atoms[-1]
		
	def atomDistance(self, atom1, atom2):
		distX = self.applyPBC(atom2.x-atom1.x, self.a)
		distY = self.applyPBC(atom2.y-atom1.y, self.b)
		distZ = self.applyPBC(atom2.z-atom1.z, self.c)
		return math.sqrt(math.pow(distX, 2)+math.pow(distY, 2)+math.pow(distZ, 2))
	
	def wrapIntoBox(self, x, y, z):
		while x>self.a:
			x -= self.a
		while y>self.b:
			y -= self.b
		while z>self.c:
			z -= self.c
		while x<0:
			x += self.a
		while y<0:
			y += self.b
		while z<0:
			z += self.c
		return x, y, z
		
	def applyPBC(self, coordinate, cellParameter):
		while coordinate<-0.5*cellParameter:
			coordinate += cellParameter
		while coordinate>0.5*cellParameter:
			coordinate -= cellParameter
		return coordinate
		
def readPDBDir(dirPath):
	pdbList = []
	for root, dirs, files in os.walk(dirPath):
		for filename in files:
			pdbFilePath = os.path.join(root, filename)
			if "box_01" in pdbFilePath:
				pdbList.append(pdbFilePath)
	return pdbList
	
def listProjectSubdirs(projectDir):
	subdirList = []
	for dirName in os.listdir(projectDir):
		dirNamePath = os.path.join(projectDir, dirName)
		if os.path.isdir(dirNamePath) and ("kpa" in dirNamePath or "kPa" in dirNamePath):
			subdirList.append(dirNamePath)
	return subdirList
	
def load_structure(source_structure_file):
	f = open(source_structure_file, "r")
	cell_a = 0.0
	cell_b = 0.0
	cell_c = 0.0
	cell_alpha = 0.0
	cell_beta = 0.0
	cell_gamma = 0.0
	source_structure = None
	for line_num, line in enumerate(f):
		if line_num<7:
			line_split = line.split()
			if len(line_split)==2:
				cell_parameter_name = line_split[0]
				cell_parameter = line_split[1]
				if cell_parameter_name=="a":
					cell_a = copy.deepcopy(float(cell_parameter))
				elif cell_parameter_name=="b":
					cell_b = copy.deepcopy(float(cell_parameter))
				elif cell_parameter_name=="c":
					cell_c = copy.deepcopy(float(cell_parameter))
				elif cell_parameter_name=="alpha":
					cell_alpha = copy.deepcopy(float(cell_parameter))
				elif cell_parameter_name=="beta":
					cell_beta = copy.deepcopy(float(cell_parameter))
				elif cell_parameter_name=="gamma":
					cell_gamma = copy.deepcopy(float(cell_parameter))
		else:
			if not source_structure:
				source_structure = OrthorhombicCrystal(cell_a, cell_b, cell_c, cell_alpha, cell_beta, cell_gamma)
			line_split = line.split()
			if len(line_split)==5:
				source_structure.addAtom(line_split[0], float(line_split[1]), float(line_split[2]), float(line_split[3]))
	f.close()
	return source_structure
	
def load_pdb(pdb_filename, cell_a, cell_b, cell_c, cell_alpha, cell_beta, cell_gamma):
	pdb_structure = OrthorhombicCrystal(cell_a, cell_b, cell_c, cell_alpha, cell_beta, cell_gamma)
	f = open(pdb_filename, "r")
	for line in f:
		lineSplit = line.split()
		if len(lineSplit)==11 and lineSplit[2]!="-":
			pdb_structure.addAtom(lineSplit[2], float(lineSplit[4]), float(lineSplit[5]), float(lineSplit[6]))
	f.close()
	return pdb_structure
	
def atoms_distance(x1, y1, z1, x2, y2, z2, a, b, c):
	distance = 0.0
	distX = applyPBC(x2-x1, a)
	distY = applyPBC(y2-y1, b)
	distZ = applyPBC(z2-z1, c)
	distance = math.sqrt(math.pow(distX, 2)+math.pow(distY, 2)+math.pow(distZ, 2))
	return distance
	
def applyPBC(coordinate, cellParameter):
		while coordinate<-0.5*cellParameter:
			coordinate += cellParameter
		while coordinate>0.5*cellParameter:
			coordinate -= cellParameter
		return coordinate
	
def calc_monolayer(source_structure_file, pdb_filename, tolerance):
	source_structure = load_structure(source_structure_file)	
	pdb_structure = load_pdb(pdb_filename, source_structure.a, source_structure.b, source_structure.c, source_structure.alpha, source_structure.beta, source_structure.gamma)
	
	del_indices = []
	for pdb_index, pdb_atom in enumerate(pdb_structure.atoms):
		for source_atom in source_structure.atoms:
			distance = atoms_distance(pdb_atom.x, pdb_atom.y, pdb_atom.z, source_atom.x, source_atom.y, source_atom.z, pdb_structure.a, pdb_structure.b, pdb_structure.c)
			#if round(pdb_atom.x, 1)==round(source_atom.x, 1) and round(pdb_atom.y, 1)==round(source_atom.y, 1) and round(pdb_atom.z, 1)==round(source_atom.z, 1):
			if distance<0.5:	
				del_indices.append(pdb_index)
				break
			
	del_indices.sort(reverse=True)
	for del_index in del_indices:
		del pdb_structure.atoms[del_index]
	
	distanceMatrix = np.zeros((len(pdb_structure.atoms), len(pdb_structure.atoms)), dtype="float32")	
	for i in range(len(pdb_structure.atoms)):
		for j in range(i, len(pdb_structure.atoms)):
			if i!=j:
				distance = atoms_distance(pdb_structure.atoms[i].x, pdb_structure.atoms[i].y, pdb_structure.atoms[i].z, pdb_structure.atoms[j].x, pdb_structure.atoms[j].y, pdb_structure.atoms[j].z, pdb_structure.a, pdb_structure.b, pdb_structure.c)
				distanceMatrix[i][j] = distanceMatrix[j][i] = distance
		
	adjacencyMatrix = np.zeros((len(pdb_structure.atoms), len(pdb_structure.atoms)), dtype="bool")
	for i in range(len(pdb_structure.atoms)):
		for j in range(i, len(pdb_structure.atoms)):
			if distanceMatrix[i][j]<(covRadius[pdb_structure.atoms[i].elementSymbol]+covRadius[pdb_structure.atoms[j].elementSymbol]+0.56):
				if i!=j:
					adjacencyMatrix[i][j] = adjacencyMatrix[j][i] = True
					
	in_monolayer = []
	monoloyer_particles = 0

	for i in range(len(pdb_structure.atoms)):
		for j in range(len(source_structure.atoms)):
			distance = atoms_distance(pdb_structure.atoms[i].x, pdb_structure.atoms[i].y, pdb_structure.atoms[i].z, source_structure.atoms[j].x, source_structure.atoms[j].y, source_structure.atoms[j].z, pdb_structure.a, pdb_structure.b, pdb_structure.c)
			if distance<(vdwRadius[pdb_structure.atoms[i].elementSymbol]+vdwRadius[source_structure.atoms[j].elementSymbol]+tolerance):
				if i not in in_monolayer:
					monoloyer_particles += 1
					in_monolayer.append(i)
					for connected_index, connected_status in enumerate(adjacencyMatrix[:,i]):
						if connected_status:
							in_monolayer.append(connected_index)
							for connected_index2, connected_status2 in enumerate(adjacencyMatrix[:,connected_index]):
								if connected_status2:
									in_monolayer.append(connected_index2)
				break
					
	#print len(pdb_structure.atoms)
	#print len(in_monolayer)
	#print monoloyer_particles
	#print adjacencyMatrix
	return monoloyer_particles
	
def write_csv(write_list, csv_filename):
	write_list = list(write_list)
	f = open(csv_filename, "w")
	for i in range(len(write_list)):
		for j in range(len(write_list[i])):
			write_list[i][j] = str(write_list[i][j])
	for write_line in write_list:
		write_string = ",".join(write_line)+"\n"
		f.write(write_string)
	f.close()
	print "Written",csv_filename

projectDir = "./dirname"
source_structure_file = "structures/filename.txt"
tolerance = 0.5 #in A
startSteps = 5*10**6

p_load = []
for projectSubdir in listProjectSubdirs(projectDir):
	monolayerAtomsList = [] 
	pressure = float(projectSubdir.split("_")[-2]+"."+projectSubdir.split("_")[-1].replace("kpa", ""))
	print pressure
	for pdbFile in readPDBDir(projectSubdir):
		monoloyerAtoms = 0
		if float(pdbFile.split("box_01_step_")[1].split(".pdb")[0])>=startSteps:
			monolayerAtomsList.append(calc_monolayer(source_structure_file, pdbFile, tolerance))
			print monolayerAtomsList[-1]
	print "--"
	monolayerAtomsSubdir = np.average(monolayerAtomsList)	
	print monolayerAtomsSubdir
	p_load.append([pressure, monolayerAtomsSubdir])
	print "--"		
	
p_load = np.array(p_load)
#p_load[:,1] = p_load[:,1]/8 #Bei Zr-fum 2x2x2
p_load = p_load[np.argsort(p_load[:, 0])]
plt.plot(p_load[:,0], p_load[:,1])
p_load = p_load.tolist()
write_csv(p_load, "filename.txt")