import pylab
import numpy
import scipy
import scipy.optimize
import scipy.linalg
import os.path
import sys

vdw_radius = {"H": 1.20, "D": 1.20, "He": 1.40, "Li": 1.82, "Be": 0.63, "B": 1.75, "C": 1.775, "N": 1.50, "O": 1.45, "F": 1.47, "Ne": 1.54, "Na": 2.27, "Mg": 1.73, "Al": 1.50, "Si": 2.10, "P": 1.90, "S": 1.80, "Cl": 1.75, "Ar": 1.88, "K": 2.75, "Ca": 1.95, "Sc": 1.32, "Ti": 1.95, "V": 1.06, "Cr": 1.13, "Mn": 1.19, "Fe": 1.26, "Co": 1.13, "Ni": 1.63, "Cu": 1.40, "Zn": 1.39, "Ga": 1.87, "Ge": 1.48, "As": 0.83, "Se": 1.90, "Br": 1.85, "Kr": 2.02, "Rb": 2.65, "Sr": 2.02, "Y": 1.61, "Zr": 1.42, "Nb": 1.33, "Mo": 1.75, "Tc": 2.00, "Ru": 1.20, "Rh": 1.22, "Pd": 1.63, "Ag": 1.72, "Cd": 1.58, "In": 1.93, "Sn": 2.17, "Sb": 1.12, "Te": 1.26, "I": 1.98, "Xe": 2.16, "Cs": 3.01, "Ba": 2.41, "La": 1.83, "Ce": 1.86, "Pr": 1.62, "Nd": 1.79, "Pm": 1.76, "Sm": 1.74, "Eu": 1.96, "Gd": 1.69, "Tb": 1.66, "Dy": 1.63, "Ho": 1.61, "Er": 1.59, "Tm": 1.57, "Yb": 1.54, "Lu": 1.53, "Hf": 1.40, "Ta": 1.22, "W": 1.26, "Re": 1.30, "Os": 1.58, "Ir": 1.22, "Pt": 1.72, "Au": 1.66, "Hg": 1.55, "Tl": 1.96, "Pb": 2.02, "Bi": 1.73, "Po": 1.21, "At": 1.12, "Rn": 2.30, "Fr": 3.24, "Ra": 2.57, "Ac": 2.12, "Th": 1.84, "Pa": 1.60, "U": 1.75, "Np": 1.71, "Pu": 1.67, "Am": 1.66, "Cm": 1.65, "Bk": 1.64, "Cf": 1.63, "Es": 1.62, "Fm": 1.61, "Md": 1.60, "No": 1.59, "Lr": 1.58}

atoms = []
cryst_a = 0
cryst_b = 0
cryst_c = 0
cryst_alpha = 0
cryst_beta = 0
cryst_gamma = 0

def read_cif(i_filename):
	global atoms, cryst_a, cryst_b, cryst_c, cryst_alpha, cryst_beta, cryst_gamma
	ins = open(i_filename, "r")
	check_coord = 0
	atom_cnt = 0
	for line in ins:
		if "_cell_length_a" in line:
			cryst_a = float(line.split()[1])
		if "_cell_length_b" in line:
			cryst_b = float(line.split()[1])
		if "_cell_length_c" in line:
			cryst_c = float(line.split()[1])
		if "_cell_angle_alpha" in line:
			cryst_alpha = float(line.split()[1])
		if "_cell_angle_beta" in line:
			cryst_beta = float(line.split()[1])
		if "_cell_angle_gamma" in line:
			cryst_gamma = float(line.split()[1])
		if ((check_coord == 1) and ("loop_" in line)):
			check_coord = 0
		if check_coord == 1:
			atoms.append([(line.split()[1]+str(atom_cnt)),line.split()[1],numpy.array([float(line.split()[2])*cryst_a,float(line.split()[3])*cryst_b,float(line.split()[4])*cryst_a])])
			atom_cnt += 1
		if "_atom_site_occupancy" in line:
			check_coord = 1
			
def smalltest_distance(xyz):
	global atoms, cryst_a, cryst_b, cryst_c, cryst_alpha, cryst_beta, cryst_gamma, vdw_radius
	xyz[0] = wrap_into_box(xyz[0], cryst_a)
	xyz[1] = wrap_into_box(xyz[1], cryst_b)
	xyz[2] = wrap_into_box(xyz[2], cryst_c)
	distances = []
	for atom in atoms:
		delta_x = xyz[0]-atom[2][0]
		delta_y = xyz[1]-atom[2][1]
		delta_z = xyz[2]-atom[2][2]
		delta_x = make_periodic(delta_x, cryst_a)
		delta_y = make_periodic(delta_y, cryst_b)
		delta_z = make_periodic(delta_z, cryst_c)
		distance = numpy.sqrt((delta_x**2)+(delta_y**2)+(delta_z**2))-vdw_radius[atom[1]]
		distances.append(distance)
	distances.sort()
	return distances[0]
	
def opt_dist(xyz):
	return -1*(smalltest_distance(xyz))
	
def make_periodic(coord, cell_param):
    while (coord < -0.5*cell_param):
        coord = coord + cell_param
    while (coord > 0.5*cell_param):
        coord = coord - cell_param
    return coord
	
def wrap_into_box(coord, cell_param):
    while (coord > cell_param):
        coord = coord - cell_param
    while (coord < 0):
        coord = coord + cell_param
    return coord
	
print "******************************************************************" 
print "* Largest pore size close to a given crystallographic coordinate *" 
print "******************************************************************\n"
"""if len(sys.argv)==1:
	print "No argument given!\n"
else:
	if ".cif" in sys.argv[1]:
		cif_filename = sys.argv[1]
	else:
		print "ERROR: Not a valid CIF file." 
		exit()
if not os.path.isfile(cif_filename):
	print "ERROR: CIF file not found." 
	exit()"""
cif_filename = "filename.cif"
#a = 0.375
#a = 0.25
#a = float(input("Enter fractinal coordinate a (between 0 and 1): ")) or a
#b = 0.375
#b = 0.75
#b = float(input("Enter fractinal coordinate b (between 0 and 1): ")) or b
#c = 0.375
#c = 0.75
#c = float(input("Enter fractinal coordinate c (between 0 and 1): ")) or c
a = 0.25
b = 0.75
c = 0.5
	
read_cif(cif_filename)			
min_xyz = scipy.optimize.fmin(opt_dist, numpy.array([a*cryst_a, b*cryst_b, c*cryst_c]), xtol=10e-4, maxiter=1000, maxfun=1000,disp=0,retall=0)
a_out = format(round((min_xyz[0]/cryst_a),3), '.3f')
b_out = format(round((min_xyz[1]/cryst_b),3), '.3f')
c_out = format(round((min_xyz[2]/cryst_c),3), '.3f')
min_diam = format(round((smalltest_distance(min_xyz)*2),3), '.3f')
print "\nRESULT:" 
print "Pore diameter of "+str(min_diam)+" Angs. at a: "+str(a_out)+" b: "+str(b_out)+" c: "+str(c_out)
