import pylab
import numpy
import scipy
import scipy.optimize
import scipy.linalg
import CoolProp.CoolProp #Requires CoolProp python module (http://coolprop.org/)
import os

atomic_mass = {"Ru": 102.91, "Pd": 106.4, "Pt": 195.09, "Ni": 58.71, "Mg": 24.305, "Na": 22.99, "Nb": 92.906, "Db": 268.0, "Ne": 20.179, "Li": 6.941, "Pb": 207.2, "Re": 128.207, "Tl": 204.37, "B": 10.81, "Ra": 226.03, "Rb": 85.468, "Ti": 47.9, "Rn": 222.0, "Cd": 112.41, "Po": 209.0, "Ta": 180.95, "Be": 9.0122, "Fr": 223.0, "Te": 127.6, "Ba": 137.33, "Os": 190.2, "La": 138.9, "Bh": 272.0, "Ge": 72.59, "Zr": 91.224, "Tc": 97.0, "Fe": 55.847, "Br": 79.904, "Sr": 87.62, "Hf": 178.49, "Hg": 200.59, "He": 4.0026, "C": 12.011, "Cl": 35.453, "Rf": 265.0, "P": 30.974, "F": 18.998, "I": 126.9, "H": 1.0079, "Mo": 95.94, "v": 50.941, "Ac": 227.0, "O": 15.999, "N": 14.007, "Kr": 83.8, "Si": 28.086, "Sn": 118.69, "W": 183.84, "Y": 88.906, "Sb": 121.75, "Bi": 206.98, "Al": 26.982, "Sg": 271.0, "Se": 78.96, "Sc": 44.955912, "Zn": 65.38, "Co": 58.933, "Ag": 107.87, "Mt": 276.0, "k": 39.096, "Ir": 192.22, "S": 32.06, "Xe": 131.3, "Mn": 54.938, "As": 74.922, "Ar": 39.948, "Au": 196.97, "At": 210.0, "Ga": 69.72, "Hs": 227.0, "Cs": 132.91, "Cr": 51.996, "Ca": 40.08, "Cu": 63.546, "In": 114.82}
avogadro = 6.02214129*10**23

def list_sub_dirs(project_path):
	dir_content_list = os.listdir(project_path)
	sub_dir_list = []
	for dir_content in dir_content_list:
		if (os.path.isdir(project_path+"/"+dir_content)==1) and (("kPa" in dir_content) or ("kpa" in dir_content)):
			sub_dir_list.append(dir_content)
	return sub_dir_list
	
def parse_towhee_out(out_path, equilibrium):
	ins = open(out_path, "r")
	ensemble = None
	temperature= None
	unvt = None
	unpt = None
	ugibbs = None
	issteps = False
	numberNegativeVolumes = 0
	stepsList = []
	stepsListComplete = []
	for line in ins:
		isensemble = False
		isunvt = False
		isunpt = False
		istemp = False
		isugibbs = False
		if " ensemble: " in line:
			isensemble = True
		if isensemble:
			ensemble = line.split()[1]
		if " temperature:" in line:
			istemp = True
		if istemp:
			temperature = float(line.split()[1])
		if issteps and len(line)<5:
			issteps = False
		if issteps and " B: " in line:
			if (float(line.split()[4])>0):
				if int(line.split()[0])>=equilibrium:
					stepsList.append([int(line.split()[0]), float(line.split()[3]), float(line.split()[4]), float(line.split()[5]), int(line.split()[6])])
				stepsListComplete.append([int(line.split()[0]), float(line.split()[3]), float(line.split()[4]), float(line.split()[5]), int(line.split()[6])])
			else:
				numberNegativeVolumes += 1
		if "Move       Box   Energy [K]  Volume [A^3] Press. [kPa] Molecules" in line:
			issteps = True
		if " u (NVT Insertion)         K" in line:
			isunvt = True
		if isunvt:
			unvt = float(line.split()[5])
		if " u (NpT Insertion)         K" in line:
			isunpt = True
		if isunpt:
			unpt = float(line.split()[5])
		if " u (Gibbs Total)           K" in line:
			isugibbs = True
		if isugibbs:
			ugibbs = float(line.split()[5])
	ins.close
	return numberNegativeVolumes, temperature, unvt, unpt, ugibbs, numpy.array(stepsList), numpy.array(stepsListComplete)
	
def coolprop_density(pressure,temp,mol_coolprop_name):
	return CoolProp.CoolProp.Props("D","T",temp,"P",pressure,mol_coolprop_name)*10**-3 #Density converted from kg/L to g/mL	
	
def write_output_file(resultsArray):
	fo = open(project_path+"/pressure_density_error.csv", "w")
	for i in range(len(resultsArray)):
		x = resultsArray[i][0]
		y = resultsArray[i][2] 
		error = (y-coolprop_density(resultsArray[i][0],temperature,mol_coolprop_name))*100
		fo.write(str(x)+","+str(y)+","+str(error)+"\n")
	fo.close()
	fo = open(project_path+"/pressure_ugibbs.csv", "w")
	for i in range(len(resultsArray)):
		x = resultsArray[i][0]
		y = resultsArray[i][-1] 
		fo.write(str(x)+","+str(y)+"\n")
	fo.close()

project_path = "./dirname"
subproject_dirs = list_sub_dirs(project_path)
mass_in_g = (2*atomic_mass["N"])/avogadro
mol_coolprop_name = "Nitrogen"
isLog = True
temperature = None
pointsCoolPropArray = 500
equilibrium = 10**6

resultsArray = []

for subproject_dir in subproject_dirs:
	out_path = project_path+"/"+subproject_dir+"/out"
	pressure = subproject_dir.replace("kPa", "")
	pressure = float(pressure.split("_")[-2]+"."+pressure.split("_")[-1])
	numberNegativeVolumes, temperature, unvt, unpt, ugibbs, stepsList, stepsListComplete = parse_towhee_out(out_path, equilibrium)
	if numberNegativeVolumes>0:
		print "WARNING: "+str(numberNegativeVolumes)+"x negative Volume!"
		print "Omitting pressure "+str(pressure)+" kPa"
		print "-"*5
	else:
		densityArray = (stepsList[:,4]*mass_in_g)/(stepsList[:,2]*10**-24)
		densityPlotArray = (stepsListComplete[:,4]*mass_in_g)/(stepsListComplete[:,2]*10**-24)
		densityAverageSimulation = numpy.average(densityArray)
		resultsArray.append([pressure, temperature, densityAverageSimulation, ugibbs])
		print "Plotting "+subproject_dir+ " ..."
		pylab.plot(stepsListComplete[:,0], densityPlotArray, linestyle='-', color='b')
		pylab.xlabel('MC steps')
		pylab.ylabel('Density / g/mL')
		pylab.title("MC steps vs. Density")
		pylab.grid(True)
		pylab.savefig(project_path+"/"+str(pressure).replace(".", "_")+"filname.png", format='png', dpi=400)
		pylab.close()
	
resultsArray = numpy.array(resultsArray)
resultsArray = numpy.sort(resultsArray, axis=0)

densityCoolPropArray = []
pressureArray = []
if isLog:
	pressureArray = numpy.logspace(numpy.log10(resultsArray[0][0]), numpy.log10(resultsArray[-1][0]), num=pointsCoolPropArray)
else:
	pressureArray = numpy.linspace(resultsArray[0][0], resultsArray[-1][0], num=pointsCoolPropArray)
for pressureCurr in pressureArray:
	densityCoolPropArray.append([pressureCurr, temperature, coolprop_density(pressureCurr,temperature,mol_coolprop_name)])
densityCoolPropArray = numpy.array(densityCoolPropArray)

print "Writing output file ..."
write_output_file(resultsArray)

print "Plotting Pressure vs. Density ..."
pylab.plot(resultsArray[:,0], resultsArray[:,2], marker='o', linestyle='--', color='b', label='Simulation')
pylab.plot(densityCoolPropArray[:,0], densityCoolPropArray[:,2], linestyle='-', color='r', label='CoolProp')

pylab.xlabel('Pressure / kPa')
pylab.ylabel('Density / g/mL')
pylab.legend(loc='lower right')
pylab.title("Pressure vs. Density")
pylab.grid(True)
pylab.savefig(project_path+"/pressure_vs_density_lin.png", format='png', dpi=400)

pylab.gca().set_xscale('log') #Logarithmic isotherm
pylab.gca().set_yscale('log') #Logarithmic isotherm
pylab.xlabel('Pressure / kPa')
pylab.ylabel('Density / g/mL')
pylab.legend(loc='upper left')
pylab.title("Pressure vs. Density")
pylab.grid(True)
pylab.savefig(project_path+"/pressure_vs_density_log.png", format='png', dpi=400)
print "Done."
pylab.close()